@@ -129,8 +129,10 @@ gst_template_match_base_init (gpointer gclass)
       "Performs template matching on videos and images, providing detected positions via bus messages.",
       "Noam Lewis <jones.noamle@gmail.com>");
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
 }
 
 /* initialize the templatematch's class */
