@@ -72,7 +72,7 @@ enum
 {
   PROP_0,
   PROP_DATA,
-  PROP_FILENAME,
+  PROP_LOCATION,
   PROP_FIT_TO_FRAME,
   PROP_X,
   PROP_Y,
@@ -164,6 +164,8 @@ gst_rsvg_overlay_set_svg_data (GstRsvgOverlay * overlay, const gchar * data,
         overlay->svg_width = svg_dimension.width;
         overlay->svg_height = svg_dimension.height;
         gst_base_transform_set_passthrough (btrans, FALSE);
+        GST_INFO_OBJECT (overlay, "updated SVG, %d x %d", overlay->svg_width,
+            overlay->svg_height);
       }
     }
   }
@@ -184,7 +186,7 @@ gst_rsvg_overlay_set_property (GObject * object, guint prop_id,
           FALSE);
       break;
     }
-    case PROP_FILENAME:
+    case PROP_LOCATION:
     {
       gst_rsvg_overlay_set_svg_data (overlay, g_value_get_string (value), TRUE);
       break;
@@ -336,7 +338,7 @@ gst_rsvg_overlay_data_sink_event (GstPad * pad, GstEvent * event)
       GST_RSVG_UNLOCK (overlay);
     }
 
-    case GST_EVENT_FLUSH_START:
+    case GST_EVENT_FLUSH_STOP:
       gst_adapter_clear (overlay->adapter);
       break;
 
@@ -436,10 +438,11 @@ gst_rsvg_overlay_stop (GstBaseTransform * btrans)
 
   if (overlay->handle) {
     g_object_unref (overlay->handle);
-    g_object_unref (overlay->adapter);
     overlay->handle = NULL;
   }
 
+  gst_adapter_clear (overlay->adapter);
+
   return TRUE;
 }
 
@@ -448,11 +451,12 @@ gst_rsvg_overlay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &video_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &data_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&data_sink_template));
 
   gst_element_class_set_details_simple (element_class, "RSVG overlay",
       "Filter/Editor/Video",
@@ -473,7 +477,7 @@ gst_rsvg_overlay_class_init (GstRsvgOverlayClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DATA,
       g_param_spec_string ("data", "data", "SVG data.", "",
           G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FILENAME,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LOCATION,
       g_param_spec_string ("location", "location", "SVG file location.", "",
           G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FIT_TO_FRAME,
