@@ -88,6 +88,10 @@
 #include <stdio.h>
 #include <string.h>
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <gst/video/video.h>
 
 #include "mpegtsmux.h"
@@ -108,7 +112,7 @@ enum
 };
 
 static GstStaticPadTemplate mpegtsmux_sink_factory =
-    GST_STATIC_PAD_TEMPLATE ("sink_%d",
+    GST_STATIC_PAD_TEMPLATE ("sink_%u",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS ("video/mpeg, "
@@ -166,10 +170,10 @@ mpegtsmux_base_init (gpointer g_class)
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegtsmux_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &mpegtsmux_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegtsmux_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mpegtsmux_src_factory));
 
   gst_element_class_set_details_simple (element_class,
       "MPEG Transport Stream Muxer", "Codec/Muxer",
@@ -288,6 +292,7 @@ mpegtsmux_dispose (GObject * object)
     g_list_free (mux->streamheader);
     mux->streamheader = NULL;
   }
+  gst_event_replace (&mux->force_key_unit_event, NULL);
   GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
@@ -661,27 +666,6 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
 
 #define COLLECT_DATA_PAD(collect_data) (((GstCollectData *)(collect_data))->pad)
 
-static MpegTsPadData *
-find_pad_data (MpegTsMux * mux, GstPad * pad)
-{
-  GSList *walk;
-  MpegTsPadData *ts_data = NULL;
-
-  GST_COLLECT_PADS_PAD_LOCK (mux->collect);
-  walk = mux->collect->abidata.ABI.pad_list;
-  while (walk) {
-    if (((GstCollectData *) walk->data)->pad == pad) {
-      ts_data = (MpegTsPadData *) walk->data;
-      break;
-    }
-
-    walk = g_slist_next (walk);
-  }
-  GST_COLLECT_PADS_PAD_UNLOCK (mux->collect);
-
-  return ts_data;
-}
-
 static gboolean
 mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -690,7 +674,7 @@ mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   gboolean forward = TRUE;
 
-  ts_data = find_pad_data (mux, pad);
+  ts_data = (MpegTsPadData *) gst_pad_get_element_private (pad);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_CUSTOM_DOWNSTREAM:
@@ -731,6 +715,8 @@ mpegtsmux_sink_event (GstPad * pad, GstEvent * event)
 out:
   if (forward)
     res = ts_data->eventfunc (pad, event);
+  else
+    gst_event_unref (event);
 
   gst_object_unref (mux);
   return res;
@@ -740,7 +726,7 @@ static gboolean
 mpegtsmux_src_event (GstPad * pad, GstEvent * event)
 {
   MpegTsMux *mux = GST_MPEG_TSMUX (gst_pad_get_parent (pad));
-  gboolean res = TRUE;
+  gboolean res = TRUE, forward = TRUE;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_CUSTOM_UPSTREAM:
@@ -755,6 +741,8 @@ mpegtsmux_src_event (GstPad * pad, GstEvent * event)
       if (!gst_video_event_is_force_key_unit (event))
         break;
 
+      forward = FALSE;
+
       gst_video_event_parse_upstream_force_key_unit (event,
           &running_time, &all_headers, &count);
 
@@ -795,15 +783,18 @@ mpegtsmux_src_event (GstPad * pad, GstEvent * event)
             break;
         }
       }
-
-      gst_event_unref (event);
+      gst_iterator_free (iter);
       break;
     }
     default:
-      res = gst_pad_event_default (pad, event);
       break;
   }
 
+  if (forward)
+    res = gst_pad_event_default (pad, event);
+  else
+    gst_event_unref (event);
+
   gst_object_unref (mux);
   return res;
 }
@@ -1348,5 +1339,5 @@ plugin_init (GstPlugin * plugin)
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR, GST_VERSION_MINOR,
-    "mpegtsmux", "MPEG-TS muxer",
+    mpegtsmux, "MPEG-TS muxer",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
