@@ -421,7 +421,7 @@ didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
   [bufQueueLock lockWhenCondition:HAS_BUFFER_OR_STOP_REQUEST];
   if (stopRequest) {
     [bufQueueLock unlock];
-    return GST_FLOW_WRONG_STATE;
+    return GST_FLOW_FLUSHING;
   }
 
   sbuf = (CMSampleBufferRef) [bufQueue lastObject];
@@ -567,7 +567,8 @@ gst_avf_video_src_base_init (gpointer gclass)
       "Reads frames from an iOS AVFoundation device",
       "Ole André Vadla Ravnås <oravnas@cisco.com>");
 
-  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_template));
 }
 
 static void
